#ifndef INTERPOLATION_SCHEMES_H
#define INTERPOLATION_SCHEMES_H

#include <Core/Interpolation/CUBIC_INTERPOLATION_FACE.h>
#include <Core/Interpolation/CUBIC_SPLINE_INTERPOLATION_FACE.h>
#include <Core/Interpolation/CURL_FREE_INTERPOLATION.h>
#include <Core/Interpolation/CURL_FREE_INTERPOLATION_4TH_ORDER.h>
#include <Core/Interpolation/CURL_FREE_INTERPOLATION_C1.h>
#include <Core/Interpolation/DIV_FREE_BALSARA.h>
#include <Core/Interpolation/DIV_FREE_BALSARA_WENO.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION_4TH_ORDER.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION_C1.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION_C1_INTERP.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION_FLUXES.h>
#include <Core/Interpolation/DIV_FREE_INTERPOLATION_INTERP.h>
#include <Core/Interpolation/INTERPOLATION_FACE.h>
#include <Core/Interpolation/LINEAR_INTERPOLATION_FACE.h>
#include <Core/Log/DEBUG_UTILITIES.h>
#include <cmath>

namespace PhysBAM{
template<class TV>
struct INTERPOLATION_SCHEMES
{
    typedef typename TV::SCALAR T;
    INTERPOLATION_FACE<TV,T>* interp=nullptr;
    const char* name=nullptr;
    struct
    {
        bool is_div=false;
        bool is_curl=false;
        bool need_avg=false;
    };
};

template<class TV> ARRAY_VIEW<INTERPOLATION_SCHEMES<TV>>
Interpolation_Schemes()
{
    typedef typename TV::SCALAR T;
    static INTERPOLATION_SCHEMES<TV> list[]=
    {
        {new LINEAR_INTERPOLATION_FACE<TV,T>,"linear",{}},
        {new CUBIC_INTERPOLATION_FACE<TV,T>,"cubic",{}},
        {new CUBIC_SPLINE_INTERPOLATION_FACE<TV,T>,"cubic-spline",{}},
        {new DIV_FREE_BALSARA<TV,T>,"balsara",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION<TV,T>,"div-free-c0",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION_INTERP<TV,T>,"div-free-c0-interp",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION_C1<TV,T>,"div-free-c1",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION_C1_INTERP<TV,T>,"div-free-c1-interp",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION_4TH_ORDER<TV,T>,"div-free-4th-order",{.is_div=true}},
        {new DIV_FREE_INTERPOLATION_FLUXES<TV,T>,"div-free-flux",{.is_div=true}},
        {new DIV_FREE_BALSARA_WENO<TV,T,3>,"balsara-weno-3",{.is_div=true,.need_avg=true}},
        {new CURL_FREE_INTERPOLATION<TV,T>,"curl-free-c0",{.is_curl=true}},
        {new CURL_FREE_INTERPOLATION_4TH_ORDER<TV,T>,"curl-free-4th-order",{.is_curl=true}},
        {new CURL_FREE_INTERPOLATION_C1<TV,T>,"curl-free-c1",{.is_curl=true}},
    };
    return {list,sizeof(list)/sizeof(*list)};
}
} // namespace PhysBAM

#endif

