#include <Core/Grids/FACE_ITERATOR.h>
#include <Core/Interpolation/DIV_FREE_BALSARA.h>
#include <Core/Interpolation/DIV_FREE_BALSARA_WENO.h>
#include <Core/Log/LOG.h>
#include <Core/Random_Numbers/RANDOM_NUMBERS.h>
#include "VELOCITY_FIELDS.h"

using namespace PhysBAM;

typedef double RW;
typedef double T;
typedef VECTOR<T,2> TV;
typedef VECTOR<int,TV::m> IV;

T eps=1e-6;

int main()
{
    DIV_FREE_BALSARA<TV, T> base;
    DIV_FREE_BALSARA_WENO<TV, T, 2> weno;
    RANDOM_NUMBERS<T> rand(123);

    int resolution = 16;
    GRID<TV> grid(IV::Constant_Vector(resolution), RANGE<TV>(TV(), TV()+resolution));
    ARRAY<T, FACE_INDEX<TV::m>> u(grid.Cell_Indices());
    auto field=Velocity_Field<TV>(0);
    for(FACE_ITERATOR<TV> it(grid);it.Valid();it.Next())
    {
        auto fi=it.Full_Index();
        u(fi)=field(grid.Face_X(fi))(it.Axis());
    }

    for(int i=0;i<100;i++)
    {
        TV X;
        rand.Fill_Uniform(X,grid.Ghost_Domain(1));
        TV a=base.Interpolate_Face(grid, u, X);
        TV b=weno.Interpolate_Face(grid, u, X);
        T e=(b-a).Magnitude();
        if (e > eps)
            LOG::printf("ERROR ");
        LOG::printf("Normal: %p -- WENO: %p\n", a, b);
    }
}
